`define NUM_CORE 2

module oc_256_aes_top(handsome_clk, handsome_din_top, handsome_dout_top, clk, reset);
	input handsome_clk;
	input handsome_din_top;
	output handsome_dout_top;
	input clk;
	input reset;

localparam input_width = 386;
localparam output_width = 129;
localparam reducer_log4 = 4;
localparam reducer_log4_top = 4;

wire [`NUM_CORE-1:0] load_i;
wire [`NUM_CORE-1:0] decrypt_i;
wire [`NUM_CORE-1:0] ready_o;
wire [127:0] data_i [`NUM_CORE-1:0];
wire [255:0] key_i [`NUM_CORE-1:0];
wire [127:0] data_o [`NUM_CORE-1:0];
wire [`NUM_CORE-1:0] handsome_din;
wire [`NUM_CORE-1:0] handsome_dout;

handsome_lfsr #(`NUM_CORE) input_lfsr_top (
			.clock(handsome_clk),
			.input_bit(handsome_din_top),
			.output_vector(handsome_din));

generate
begin:G1
	genvar i;
	for (i = 0; i < `NUM_CORE; i = i+1)
	begin :stamp_inst

		handsome_lfsr #(input_width) input_lfsr (
			.clock(handsome_clk),
			.input_bit(handsome_din[i]),
			.output_vector({load_i[i],decrypt_i[i],data_i[i],key_i[i]}));

		oc_256_aes oc_stamp(
			.clk(clk),
			.reset(reset),
			.load_i(load_i[i]),
			.decrypt_i(decrypt_i[i]),
			.data_i(data_i[i]),
			.key_i(key_i[i]),
			.ready_o(ready_o[i]),
			.data_o(data_o[i]));

		handsome_reducer #(reducer_log4) output_reducer (
			.clock(handsome_clk),
			.input_vector({ready_o[i],data_o[i]}),
			.output_bit(handsome_dout[i]));

	end

handsome_reducer #(reducer_log4_top) output_reducer_top (
	.clock(handsome_clk),
	.input_vector(handsome_dout[`NUM_CORE-1:0]),
	.output_bit(handsome_dout_top));

end
endgenerate
endmodule
